/*-
 * Copyright (c) 2003
 *		Stepachev Andrey <aka50@mail.ru>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/* 
 * cmd.h -- header for commands module
 *
 * $Id: cmd.h,v 1.5 2003/08/15 21:02:29 aka50 Exp $
 */

#define CMD_BUFPRMPT_RATIO 	4
#define CMD_MAX_PRMPT 		2048
#define CMD_MAX_CMD 		32
#define CMD_MAX_CHNLNM 		64
#define CMD_MAX_BUFFER 		2048
#define CMD_MAX_HISTSZ 		256

#define CMD_DIR_FWD    1
#define CMD_DIR_REV   -1

#define CMD_PRT_CHAR   0
#define CMD_PRT_END    1
#define CMD_PRT_WORD   2

#include "queue.h"

struct hist_node {
	char *val;
	TAILQ_ENTRY(hist_node) link;
};

struct cmd_line {
	int visln;

	int bufstrt;
	int bufpos;

	int prmptsz;
	int bufsz;
	char prmpt[CMD_MAX_PRMPT + 1];
	char buf[CMD_MAX_BUFFER + 1];

	
	int histsz;
	struct hist_node *curhn;
	TAILQ_HEAD(, hist_node) hist;
};

struct cmd_line *cmd_new(int isize, const char *prompt);
void cmd_free(struct cmd_line *cl);

int cmd_setvis(struct cmd_line *cmd, int size);
char *cmd_getbuf(struct cmd_line *cmd);
char *cmd_getprmpt(struct cmd_line *cmd);
int cmd_getprmptsz(struct cmd_line *cmd);
int cmd_getbufsz(struct cmd_line *cmd);
int cmd_getcurpos(struct cmd_line *cmd);
int cmd_putch(struct cmd_line *cmd, char ch);
int cmd_kill(struct cmd_line *cmd, int dir, int part);
int cmd_mvcurs(struct cmd_line *cmd, int dir, int part);
int cmd_execute(struct cmd_line *cmd);
int cmd_setnexthist(struct cmd_line *cmd, int dir);
int cmd_clear(struct cmd_line *cmd, int upto);

