/*-
 * Copyright (c) 2003
 *		Stepachev Andrey <aka50@mail.ru>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/* 
 * gui.c -- GUI (ncurses) 
 *
 * $Id: gui.c,v 1.6 2003/08/15 21:02:29 aka50 Exp $
 */

#include <sys/types.h>
#include <stdlib.h>
#include <ncurses.h>

#include "dmalloc.h"
#include "membuf.h"

#include "ichat.h"
#include "gui.h"
#include "conv_rus.h"
#include "proto.h"
#include "cmd.h"

#define GUI_MAX_LOGBUF 255

#define GUI_COLOR_NORMAL 	0
#define GUI_COLOR_SYSTEM 	1
#define GUI_COLOR_NICK   	2
#define GUI_COLOR_CHANNEL   	3

static WINDOW  *chatwin = NULL;
static int     initialized = 0;

static int  chathght = 0;
static int  chatwdth = 0;

static int  inpline  = 0;

static struct cmd_line *cmdln;

#ifndef CTRL
#define CTRL(x)         ((x) & 0x1f)
#endif

#define QUIT            CTRL('Q')
#define ESC          	CTRL('[')


int
gui_init()
{
        int bg;

	initscr(); 

	getmaxyx(stdscr, chathght, chatwdth);
	chathght -= 1;

/*  	cbreak();   */
 	raw(); 
	noecho();
	nonl();
	intrflush(stdscr, FALSE);
	nodelay(stdscr, TRUE);
	keypad(stdscr, TRUE);
        start_color();
/*         use_default_colors(); */


	bg = COLOR_BLACK;
        init_pair(GUI_COLOR_NORMAL,  COLOR_WHITE,  bg);
        init_pair(GUI_COLOR_SYSTEM,  COLOR_RED,    bg);
        init_pair(GUI_COLOR_CHANNEL, COLOR_GREEN,  bg);
        init_pair(GUI_COLOR_NICK,    COLOR_YELLOW, bg);

	chatwin = subwin(stdscr,  chathght, chatwdth, 0, 0);
	scrollok(chatwin, TRUE);
	wmove(chatwin, 0, 0);
        wattrset(chatwin, COLOR_PAIR(GUI_COLOR_NORMAL));
	werase(chatwin);

	wmove(chatwin, 0, 0);
	hline(ACS_HLINE, chatwdth);
	for (bg = 0; bg < chathght + 5; bg++)
		wprintw(chatwin, "\n");
	
	wrefresh(chatwin);

	move(chathght - 1, 0);
	hline(ACS_HLINE, chatwdth);
	scrollok(stdscr, FALSE);

	inpline = chathght;
	move(inpline, 0);

	cmdln = cmd_new(chatwdth,">");

	refresh();

	initialized = 1;
	return 0;
}

void
gui_fini()
{
	endwin();
	cmd_free(cmdln);
	initialized = 0;
	return;
}

int
gui_isactive()
{
	if (initialized)
		return 1;
	else 
		return 0;
}

int
gui_update_inpline()
{
	char *buf;
	int i, bufsz, curpos;

	move(inpline, 0);
	clrtoeol();

	cmd_setvis(cmdln, chatwdth);
	buf   = cmd_getprmpt(cmdln);
	bufsz = cmd_getprmptsz(cmdln);
	for (i = 0; i < bufsz; i++)
		wprintw(stdscr, "%c", buf[i]); 

	buf   = cmd_getbuf(cmdln);
	bufsz = cmd_getbufsz(cmdln);
	for (i = 0; i < bufsz; i++)
		wprintw(stdscr, "%c", buf[i]); 

	curpos = cmd_getcurpos(cmdln);

	move(inpline, curpos);

	return 0;
}


int
gui_handle_input(struct membuf *obuf)
{
	int ch;
	ch = getch();

	switch(ch){
	case ESC:
		ch = getch();
		break;
	case CTRL('Q'):
	case CTRL('C'):
		return -1;
	case 13:
		cmd_execute(cmdln);
		break;
	case ERR:
		break;

	case KEY_BACKSPACE:
		if (cmd_kill(cmdln, CMD_DIR_REV, CMD_PRT_CHAR))
			beep();
		break;

	case CTRL('D'):
	case KEY_DC:
		if (cmd_kill(cmdln, CMD_DIR_FWD, CMD_PRT_CHAR))
			beep();
		break;

	case CTRL('K'):
		if (cmd_kill(cmdln, CMD_DIR_FWD, CMD_PRT_END))
			beep();
		break;

	case CTRL('E'):
	case KEY_END:
		if (cmd_mvcurs(cmdln, CMD_DIR_FWD, CMD_PRT_END))
			beep();
		break;

	case CTRL('A'):
	case KEY_HOME:
		if (cmd_mvcurs(cmdln, CMD_DIR_REV, CMD_PRT_END))
			beep();
		break;

 	case CTRL('F'):
	case KEY_RIGHT:
		if (cmd_mvcurs(cmdln, CMD_DIR_FWD, CMD_PRT_CHAR))
			beep();
		break;

	case CTRL('B'):
	case KEY_LEFT:
		if (cmd_mvcurs(cmdln, CMD_DIR_REV, CMD_PRT_CHAR))
			beep();
		break;

	case KEY_UP:
		if (cmd_setnexthist(cmdln, CMD_DIR_FWD))
			beep();
		break;

	case KEY_DOWN:
		if (cmd_setnexthist(cmdln, CMD_DIR_REV))
			beep();
		break;


	default:
		ch &= 0xff;
		if (ch < 32)
			break;
		if (cmd_putch(cmdln, ch))
			beep();
		break;
	}

	gui_update_inpline();

	refresh();
	return 0;
}

int
gui_put_log(char *fmt, ...)
{
	va_list ap;

	move(chathght - 1, 0);
	wclrtoeol(chatwin);
	scrollok(chatwin, TRUE);
	
	wattrset(chatwin, COLOR_PAIR(GUI_COLOR_SYSTEM));
	wprintw(chatwin, "System: ");
	va_start(ap, fmt);
	vwprintw(chatwin, fmt, ap);
	va_end(ap);
	wprintw(chatwin, "\n");
	wattrset(chatwin, COLOR_PAIR(GUI_COLOR_NORMAL));
	scrollok(chatwin, FALSE);
	hline(ACS_HLINE, chatwdth);
	wrefresh(chatwin);
	gui_update_inpline();
	refresh();

	return 0;
}

int
gui_put_chat(int msgtype, const char *from, const char *line,  char *fmt, ...)
{
	va_list ap;

	move(chathght - 1, 0);
	wclrtoeol(chatwin);
	scrollok(chatwin, TRUE);
	

	if (!strcmp(line, ICHAT_COMMON_LINE)){
		wattrset(chatwin, COLOR_PAIR(GUI_COLOR_NICK));
		wprintw(chatwin, "<%s>: ", from);
	} else {
		wattrset(chatwin, COLOR_PAIR(GUI_COLOR_CHANNEL));
		wprintw(chatwin, "[%s] ", line);
		wattrset(chatwin, COLOR_PAIR(GUI_COLOR_NICK));
		wprintw(chatwin, "<%s>: ", from);
	}
	wattrset(chatwin, COLOR_PAIR(GUI_COLOR_NORMAL));
	va_start(ap, fmt);
	vwprintw(chatwin, fmt, ap);
	va_end(ap);
	wprintw(chatwin, "\n");
	scrollok(chatwin, FALSE);
	hline(ACS_HLINE, chatwdth);
	wrefresh(chatwin);
	gui_update_inpline();
	refresh();

	return 0;
}
