/*-
 * Copyright (c) 2003
 *		Stepachev Andrey <aka50@mail.ru>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*
 * membuf.c -- Buffers support routines
 * 
 * $Id: membuf.h,v 1.2 2003/07/07 18:06:42 aka50 Exp $
 */
#ifndef _MEMBUF_H_
#define _MEMBUF_H_

#include <sys/types.h>

#include <stdlib.h>
#include <stdarg.h>

#define MEM_OK		  0
#define MEM_ERRNO 	(-1)
#define MEM_EINVAL 	(-2)

/*
 * Macros for membufs manipulations
 * mbtod(mb,t) - convert membuf pointer to data pointer of correct type
 * mbtor(mb,t) - convert membuf pointer to reserve data pointer of correct type
 * mbtosz(mb)  - convert membuf pointer to size of contained data
 * mbtorsz(mb) - convert membuf pointer to size of reserved space
 */

#define mbtod(mb,t) 	((t)((mb)->mb_fb))
#define mbtor(mb,t)	((t)((mb)->mb_fb + (mb)->mb_size))
#define mbtosz(mb)	((mb)->mb_size)
#define mbtorsz(mb)	((mb)->mb_rsrv)

struct membuf {
	u_int32_t 	mb_magic;
#define MEMBUF_MAGIC  	0xf3fb32c1 

	char   *mb_fb;			/* begin of buffer */
	size_t	mb_size;	        /* size of buffer */
	size_t  mb_rsrv;		/* reserved size in buffer */

	/* internal data */
	size_t  mb_pshft;		/* user defined page size */
#define MEMBUF_DEFAULT_PSHFT 12U
	size_t	mb_psize;		/* allocated memory size */	
	char   *mb_pdata;		/* allocated memory page */
};

extern int membuf_balance;

#define mb_expand(mb, diff) mb_resize((mb), mbtosz(mb) + (diff))
#define mb_append_str(mb, str) mb_append((mb), (str), strlen(str))
#define mb_append_mb(mb1, mb2) mb_append((mb1), mbtod((mb2), char *), mbtosz((mb2)))
#define mb_put_str(mb, str) mb_put((mb), (str), strlen(str))

int  mb_init		__P((struct membuf *mb));
int  mb_reinit         	__P((struct membuf *mb, int def));
void mb_free		__P((struct membuf *mb));
int  mb_reset		__P((struct membuf *mb, size_t size));
int  mb_resize		__P((struct membuf *mb, size_t size));
int  mb_sprintf		__P((struct membuf *mb, const char *fmt, ...));
int  mb_vsprintf	__P((struct membuf *mb, const char *fmt, va_list ap));
int  mb_put		__P((struct membuf *mb, const void *data, 
			     size_t size));
int  mb_append		__P((struct membuf *mb, const void *data, 
			     size_t size));
int  mb_append_str	__P((struct membuf *mb, const void *data ));
int  mb_append_strfmt   __P((struct membuf *mb, const char *fmt, ...));
int  mb_append_vstrfmt	__P((struct membuf *mb, const char *fmt, va_list ap));

int  mb_consume		__P((struct membuf *mb, size_t size));
int  mb_consume_to_mb 	__P((struct membuf *mb_src, struct membuf *mb_dst, 
			     size_t size));
int  mb_exchange_mb	__P((struct membuf *mb1, struct membuf *mb2));
int  mb_set_pshft	__P((struct membuf *mb, size_t shift));
int  mb_set_rsrv	__P((struct membuf *mb, size_t size));
int  mb_append_rsrv	__P((struct membuf *mb, size_t size));

#endif /* _MEMBUF_H_ */
