/*-
 * Copyright (c) 2003
 *		Stepachev Andrey <aka50@mail.ru>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*
 * proto.c -- IChat protocol routines implementation
 * 
 * $Id: proto.c,v 1.7 2003/07/11 13:42:38 aka50 Exp $
 */

#include <sys/types.h>
#include <stdlib.h>

#include "dmalloc.h"
#include "membuf.h"
#include "strops.h"
#include "rc4.h"
#include "ichat.h"
#include "proto.h"
#include "conv_rus.h"
#include "gui.h"
#include "user.h"

#define SEP		"\x13"
#define DSEP		SEP SEP
#define TOALL		"*"

static int msgcnt = 1;

static int ich_cmd_common(struct membuf *mb, const char *cmd);
static int ich_cmd_uinfo(struct membuf *mb, const char *line, const char *msg_to, const char *statusmsg, int status);


int
ich_pkt_make(struct membuf *mb, const char *msg_to, struct membuf *payload)
{
	size_t inlen, totlen;
	struct rc4_state rs;

	totlen = strlen(signature) + 1 + strlen("FORWARD") + 1 + strlen(msg_to) + 1 + mbtosz(payload);
	mb_append_strfmt(mb, "%u%c%s%c%s%c%s%c", totlen, 0, signature, 0, "FORWARD", 0, msg_to, 0);
	inlen = mbtosz(mb);
	mb_append_mb(mb, payload);
	rc4_setup(&rs, "tahci", 5);
	rc4_crypt(&rs, mbtod(mb, u_char *) + inlen, mbtosz(payload));
	return 0;
}

/* 
 * Parses packet from specified buffer... returns:
 * 0 - no message
 * 1 - have message
 * < 0 - error 
 */

int
ich_pkt_parse(struct membuf *mb, struct membuf *payload)
{
	struct rc4_state rs;
	int inlen, lenlen;

	/* here can be some caveeats... if server sends only one character... 
	 * I will thing that it is length... so... i shall think about it
	 */

	mb_reset(payload, 0);

	inlen = atoi(mbtod(mb, const char *));
	if (inlen > ICHAT_MAX_MSGSZ || inlen < ICHAT_MIN_MSGSZ){
		ich_log(ICHAT_LOG_CRIT, "Wrong message size %d, possible wrong protocol !!!(%s)", inlen, mbtod(mb, char *));
		return -1;
	}
	lenlen = strlen(mbtod(mb, char *)) + 1;

	if (inlen  + lenlen <= mbtosz(mb)){
		/* we got full message... parse it */
		if (strcmp(mbtod(mb, char *) + lenlen, "FORWARD")){
			ich_log(ICHAT_LOG_CRIT, "Wrong command %s, possible wrong protocol !!!", 
				mbtod(mb, char *)+lenlen);
			return -1;
		}
		mb_consume(mb, 8 + lenlen);
		inlen -= 8 ;
		rc4_setup(&rs, "tahci", 5);
		rc4_crypt(&rs, mbtod(mb, u_char *), inlen);
		mb_consume_to_mb(mb, payload, inlen);
		return 1;
	}

	return 0;
}

static int
ich_cmd_common(struct membuf *mb, const char *cmd)
{
	mb_reset(mb, 0);
	mb_append_strfmt(mb, "%s%s%s%u%s%s%s%s%s", SEP, "iChat", DSEP, msgcnt++, DSEP, signature, DSEP, cmd, SEP);
	return 0;
}

static int
ich_cmd_uinfo(struct membuf *mb, const char *line, const char *msg_to, const char *statusmsg, int status)
{
	mb_append_strfmt(mb, "%s%s%s%s%s%s%s%s%s%s%s%s%s%u%s",
			 SEP, line, DSEP, login, DSEP, nickname, DSEP DSEP, 
			 statusmsg, DSEP, msg_to, DSEP, ICHAT_VER, DSEP, status, SEP);
	return 0;
}

int
ich_cmd_disconnect(struct membuf *mb, const char *line)
{
	ich_cmd_common(mb, "DISCONNECT");
	mb_append_strfmt(mb, "%s%s%s", SEP, line, SEP);
	return 0;
}

int
ich_cmd_connect(struct membuf *mb, const char *line, const char *msg_to, const char *statusmsg, int status)
{
	ich_cmd_common(mb, "CONNECT");
	ich_cmd_uinfo(mb, line, msg_to, statusmsg, status);
	return 0;
}

int
ich_cmd_refresh(struct membuf *mb, const char *line, const char *msg_to, const char *statusmsg, int status)
{
	ich_cmd_common(mb, "REFRESH");
	ich_cmd_uinfo(mb, line, msg_to, statusmsg, status);
	return 0;
}

int
ich_cmd_text(struct membuf *mb, const char *line, const char *msg_to, const char *msg_txt)
{
	ich_cmd_common(mb, "TEXT");
	mb_append_strfmt(mb, "%s%s%s%s%s%s%s", SEP, line, DSEP, msg_txt, DSEP, msg_to, SEP);
	return 0;
}

int
ich_cmd_status(struct membuf *mb, const char *away_msg, int status)
{
	ich_cmd_common(mb, "STATUS");
	mb_append_strfmt(mb, "%s%d%s%s%s", SEP, status, DSEP, away_msg, SEP);
	return 0;
}

int
ich_rpl_parse(struct membuf *mb)
{
#define MAXARGS 12
	char *av[MAXARGS];
	int ac, i;
	char *p;

	ac = splitline(mbtod(mb, char *), av, MAXARGS, "\x13");
	for (i = 0; i< ac; i++){
		conv_winkoi(av[i]);
		p = av[i];
		while(*p){
			if (*p == '\n' || *p == '\r')
				*p = ' ';
			p++;
		}
		ich_log(ICHAT_LOG_DEBUG2, "arg[%d]: %s", i, av[i]);
	}
	if (ac < 4) {
		ich_log(ICHAT_LOG_CRIT, "Insufficient args: %d", ac);
		return 0;
	}

	if (strcmp(av[0], "iChat")){
		ich_log(ICHAT_LOG_CRIT, "Incorrect magic.");
		return 0;
	}

	if (!strcmp(av[3],"CONNECT") && ac == 11){
		user_connect(atoi(av[1]) /* cnt */, av[2] /* signature */, av[6] /* nickname */,
			     av[4] /* line */, av[9] /* ver */, av[7] /* message */, atoi(av[10]) /* state */);

	} else if (!strcmp(av[3], "DISCONNECT") && ac == 5){
		user_disconnect(atoi(av[1]) /* cnt */, av[2] /* signature */, av[4] /* line */);

	} else if (!strcmp(av[3], "TEXT") && ac == 7){ 
		/* 
		 * XXX 
		 * sometimes clients sends empty messages with garbage at the end... 
		 */
		user_text(atoi(av[1]) /* cnt */, av[2] /* signature */, av[4] /* line */, av[5] /* text */);
	} else if (!strcmp(av[3], "TEXT") && ac == 6){
		user_text(atoi(av[1]) /* cnt */, av[2] /* signature */, av[4] /* line */, av[5] /* text */);

	} else if (!strcmp(av[3], "STATUS_REQ") && ac == 4){
		user_status(atoi(av[1]) /* cnt */, av[2] /* signature */);

	} else if (!strcmp(av[3], "BOARD") && ac == 5){
		user_board(atoi(av[1]) /* cnt */, av[2] /* signature */, atoi(av[4]) /* chunk */, "");
	} else if (!strcmp(av[3], "BOARD") && ac == 6){
		user_board(atoi(av[1]) /* cnt */, av[2] /* signature */, atoi(av[4]) /* chunk */, av[5] /* text */);

	} else if (!strcmp(av[3], "RECEIVED") && ac == 6){
		user_recieved(atoi(av[1]) /* cnt */, av[2] /* signature */, av[4] /* line */, av[5] /* text */);

	} else if (!strcmp(av[3],"REFRESH") && ac == 11){
		user_refresh(atoi(av[1]) /* cnt */, av[2] /* signature */, av[6] /* nickname */,
			     av[4] /* line */, av[9] /* ver */, av[7] /* message */, atoi(av[10]) /* state */);
	} else 
		gui_put_log("<%s:%d>", av[3], ac);

	return 0;
#undef MAXARGS
}
