/*-
 * Copyright (c) 2001
 *		Stepachev Andrey <aka50@mail.ru>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*
 * user.h -- header for iChat users list operations 
 * 
 * $Id: user.h,v 1.4 2003/07/11 13:43:40 aka50 Exp $
 */

#ifndef _USER_H_
#define _USER_H_

#include <sys/types.h>

#include "queue.h"

struct user {
	TAILQ_ENTRY(user) link;

	char 	*sig;
	char 	*nick;
	char 	*ver;
	int   	 state;
	u_int    cnt;
	time_t 	 lseen; /* last message from this user */
};


int user_init();
void user_fini();

int user_lst_update();

int user_connect(u_int cnt, 
		 const char *sign, const char *nickname, const char *line, 
		 const char *ver, const char *hmsg, int state);
int user_refresh(u_int cnt, 
		 const char *sign, const char *nickname, const char *line, 
		 const char *ver, const char *hmsg, int state);
int user_disconnect(u_int cnt, const char *sign, const char *line);
int user_text(u_int cnt, const char *sign, const char *line, const char *message);
int user_status(u_int cnt, const char *sign);
int user_board(u_int cnt, const char *sign, int chunk, const char *text);
int user_recieved(u_int cnt, const char *sign, const char *line, const char *text);


int user_sys_list();

struct user *find_user(const char *sign);
struct user *find_userbynick(const char *nick);

#endif  /* _USER_H_ */
